/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_adc.h"             */
/*  Constants and definitions for ADC.                                       */
/*                                                                           */
/*  Author : JYA                        Last modification : 01/23/2020       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
            1.1  Register numbers
            1.2  Tag numbers and counts

         2  Structures and typedefs
            2.1  High level ADC driver definitions - Adc_drvdef
            2.2  Low level ADC driver interface - Adc_desc
            2.3  ADC input context - Adc_inpctx
            2.4  ADC device context - Adc_devctx

         3  Public variables
         4  Prototypes of public procedures
         5  RAM data to be included in board_xyz.c

*/


#ifndef DRV_ADC_H                      /* If this file is not already        */
#define DRV_ADC_H                      /* included, include it.              */


/*
    1    Defines
    ____________

    1.1  Register numbers
    ---------------------
*/

#define NR_ADC_TYPE               0    /* string  TYPE          (  1 UINT )  */
#define NR_ADC_NAME               1    /* string  NAME          (  2 UINTs)  */
#define NR_ADC_NUMDEV             3    /* integer NUM           (  1 UINT )  */

#define NR_ADC_UNAMES             4    /* string  UNAMES        (128 UINTs)  */
#define NR_ADC_VREF             132    /* integer VREF          (  1 UINT )  */
#define NR_ADC_LDEPTH           133    /* integer LDEPTH        (  1 UINT )  */
#define NR_ADC_MINFREQ          134    /* integer MINFREQ       (  1 UINT )  */
#define NR_ADC_MAXFREQ          135    /* integer MAXFREQ       (  1 UINT )  */

#define NR_ADC_DEPTH            136    /* integer DEPTH         (  1 UINT )  */
#define NR_ADC_FORMAT           137    /* integer FORMAT        (  1 UINT )  */
#define NR_ADC_ENDIAN           138    /* integer ENDIAN        (  1 UINT )  */
#define NR_ADC_REQSIZE          139    /* integer REQSIZE       (  1 UINT )  */
#define NR_ADC_OUTSIZE          140    /* integer OUTSIZE       (  1 UINT )  */
#define NR_ADC_SCANFREQ         141    /* integer SCANFREQ      (  1 UINT )  */
#define NR_ADC_SAMPFREQ         142    /* integer SAMPFREQ      (  1 UINT )  */

#define NR_ADC_UNAME            143    /* string  UNAME         (  2 UINT )  */

#define NR_ADC_MODE             145    /* integer MODE          (  1 UINT )  */
#define NR_ADC_IHYST            146    /* integer IHYST         (  1 UINT )  */
#define NR_ADC_SAMPLE           147    /* integer SAMPLE        (  1 UINT )  */


/*
    1.2  Tag numbers and counts
    ---------------------------
*/

#define NBTAG_ADC_DEV            15    /* Number of tags for devices.        */
#define NBTAG_ADC_SUB             4    /* Number of sub-channel tags         */
#define NBTAG_ADC  (NBTAG_ADC_DEV \
                    +NBTAG_ADC_SUB)    /* Total Number of tags               */

/*
   1.3  Flags for supported bit depth per sample
   ---------------------------------------------
*/


#define ADC_4BIT          (1U <<  0)   /* Device support 4 bit samples.      */
#define ADC_6BIT          (1U <<  2)   /* Device support 6 bit samples.      */
#define ADC_8BIT          (1U <<  4)   /* Device support 8 bit samples.      */
#define ADC_10BIT         (1U <<  6)   /* Device support 8 bit samples.      */
#define ADC_12BIT         (1U <<  8)   /* Device support 12 bit samples.     */
#define ADC_16BIT         (1U << 10)   /* Device support 16 bit samples.     */
#define ADC_20BIT         (1U << 12)   /* Device support 20 bit samples.     */
#define ADC_24BIT         (1U << 14)   /* Device support 24 bit samples.     */
#define ADC_28BIT         (1U << 16)   /* Device support 28 bit samples.     */
#define ADC_32BIT         (1U << 18)   /* Device support 32 bit samples.     */


#ifndef CACA


/*
    2  Structures and typedefs
    __________________________

    2.1  High level ADC driver definitions - Adc_drvdef
    ---------------------------------------------------
*/

typedef struct                         /* High level ADC parameters          */
  BEGIN                                /* ---------------------------------- */
    IWORD          ldevnum           ; /* Low-level driver index             */
    const CHAR    *unames            ; /* List of MUX inputs names           */
    UINT           vref              ; /* Vref value in milli-volts          */
    UINT           supp_depth        ; /* Sample depth supported by ADC      */
    UINT           supp_dma          ; /* Sample depth supported thru DMA    */
    UINT           list_sizes        ; /* Output sample sizes in bytes       */
    UINT           minfreq           ; /* ADC min sampling frequency in Hz   */
    UINT           maxfreq           ; /* ADC max sampling frequency in Hz   */
  END              Adc_drvdef        ; /* ---------------------------------- */

/*
    2.2  Low level ADC driver interface - Adc_desc
    ----------------------------------------------
*/
typedef struct                         /* Low-level procedure's addresses    */
  BEGIN                                /* ---------------------------------- */
    void           (*info       )()  ; /* Get start-up information on device */
    void           (*setpow     )()  ; /* Set power ON/OFF command           */
    void           (*setinp     )()  ; /* Select one MUX input               */
    void           (*setpar     )()  ; /* Configure ADC parameters           */
    int            (*getsmp     )()  ; /* Get a new sample                   */
    void           (*itsmp      )()  ; /* Enable or disable interrupts       */
    void           (*rcvdma     )()  ; /* Receive samples through DMA        */
  END              Adc_desc          ; /* ---------------------------------- */

/*
    2.3  ADC input context - Adc_inpctx
    -----------------------------------
*/

typedef struct                         /* ADC MUX input context              */
  BEGIN                                /* ---------------------------------- */
    Adc_desc      *desc              ; /* Low-level driver interface.        */
    USHORT         vl                ; /* Logical channel in AUT_ADC         */
                                       /* ---------------------------------- */
    UINT           iodsub            ; /* Subchannel IOD.                    */
    USHORT         reqemet           ; /* Allocator automaton.               */
    USHORT         reqevl            ; /* Allocator logical channel          */
    USHORT         reqnode           ; /* Allocator node.                    */
                                       /* ---------------------------------- */
    UCHAR          dev               ; /* High level controller number       */
                                       /* ---------------------------------- */
    UCHAR          mode              ; /* STOP SCAN SAMP                     */
    UCHAR          inext             ; /* Next input context to be polled    */
    UCHAR          ihyst             ; /* IND_REPORT Hysteresis              */
    UINT           psamp             ; /* Last sampled value                 */
  END              Adc_inpctx        ; /* ---------------------------------- */

/*
    2.4  ADC device context - Adc_devctx
    ------------------------------------
*/

typedef struct                         /* ADC device context                 */
  BEGIN                                /* ---------------------------------- */
    Req_ctx        rctx              ; /* Allocation context.   req_lib.c    */
    Key_lnk        key               ; /* Intern job queues.    req_lib.c    */
                                       /* ---------------------------------- */
    Adc_drvdef     *def              ; /* Device generic definitions.        */
    void           (*setpow    )()   ; /* ADC power ON/OFF command.          */
    void           (*setinp    )()   ; /* Select one MUX input               */
    void           (*setpar    )()   ; /* Configure ADC parameters           */
    int            (*getsmp    )()   ; /* Read a new sample.                 */
    void           (*itsmp     )()   ; /* Enable or disable interrupts.      */
    void           (*rcvdma    )()   ; /* Start a DMA transfer               */
                                       /* ---------------------------------- */
    const CHAR     *name             ; /* Hardware name string address       */
    UCHAR          depth             ; /* Output sample depth as enum        */
    UCHAR          format            ; /* Output sample format as enum       */
    UCHAR          endian            ; /* Output samples 0:CPU  1:MSB  2:LSB */
    UCHAR          reqsize           ; /* Requested size DEFAULT 1 2 3 4     */
    UINT           scanfreq          ; /* Scanning frequency in Hz           */
    UINT           sampfreq          ; /* Sampling frequency in Hz           */
                                       /* ---------------------------------- */
    Adc_inpctx     *inp_ctx          ; /* Address of MUX input context table */
    UCHAR          nbinps            ; /* Number of MUX input contexts       */
    UCHAR          nballoc           ; /* Count of allocated MUX inputs      */
    UCHAR          sfirst            ; /* First input context to scan        */
                                       /* ---------------------------------- */
    UCHAR          dma               ; /* 0:DMA not used  1:DMA used         */
    UCHAR          outsize           ; /* Output samples size in bytes 1..4  */
    CHAR           oshft             ; /* Output sample left/right shift     */
    UINT           mvmul             ; /* MilliVolts multiplier              */
    UINT           mvdiv             ; /* MilliVolts divider                 */
    UCHAR          oorder[4]         ; /* Output samples byte order          */
                                       /* ---------------------------------- */
    UCHAR          *buf              ; /* Samples receive buffer address     */
    USHORT         bufsize           ; /* Samples buffer size in bytes       */
    USHORT         ircv              ; /*                                    */
    UINT           idto              ;

  END              Adc_devctx        ; /* ---------------------------------- */


/*
    3  Public variables
    ___________________
*/

extern UCHAR       adc_numdrv        ; /* Driver identifier for board access.*/


/*
    4  Prototypes of public procedures
    __________________________________
*/

  void  itadcsmp            (IWORD                                           );
  void  itadcdma            (IWORD                                           );


/*
    5  RAM data to be included in board_xyz.c
    _________________________________________

*/

#ifdef BOARD_XYZ_DATA_C

#define ADC_NBVL (1+ADC_NBDEV+ADC_NBINP)

const USHORT adc_nbiod  = ADC_NBIOD  ; /* Controllers seen by the VMOS       */
const USHORT adc_nbvl   = ADC_NBVL   ; /* Number of LW                       */
const USHORT adc_nbdev  = ADC_NBDEV  ; /* Number of device contexts          */
const USHORT adc_nbinp  = ADC_NBINP  ; /* Number of pin context's            */

USHORT       adc_states[ADC_NBVL]    ; /* Logical Ways states.               */
Adc_devctx   adc_devctx[ADC_NBDEV]   ; /* Controller's context table.        */
Adc_inpctx   adc_inpctx[ADC_NBINP]   ; /* Pins' contexts table.              */

#endif                                 /* End of BOARD_XYZ_C                 */

#else                                  /* CACA defined                       */

#define ADC_NBREG 6

/*
    2    Structures and typedefs
    ____________________________
*/


/*
   2.1  High level ADC driver definitions - Adc_drvdef
   ---------------------------------------------------
*/

typedef struct                         /* ADC driver definitions.            */
  BEGIN                                /* ---------------------------------- */
    UINT           lctx              ; /* Get start-up information on device */
    UINT           supp_depth        ; /* Sample depth supported by device.  */
    UINT           supp_dma          ; /* Sample depth supported thru DMA.   */
  END              Adc_drvdef        ; /* ---------------------------------- */



/*
    2.2 Low level ADC driver interface - Adc_desc
    ---------------------------------------------
*/

typedef struct                         /* Low-level procedure's addresses    */
  BEGIN                                /* ---------------------------------- */
    void           (*info       )()  ; /* Get start-up information on device */
    void           (*setfmt     )()  ; /* Configure ADC parameters           */
    void           (*itsmp      )()  ; /* Enable or disable interrupts.      */
    int            (*getsmp     )()  ; /* Get a new sample.                  */
    void           (*rcvdma     )()  ; /* Receive samples through DMA.       */
  END              Adc_desc          ; /* ---------------------------------- */


/*
   2.3  ADC device context - Adc_ctx
   ---------------------------------
*/

typedef struct                         /* ADC device context                 */
  BEGIN                                /* ---------------------------------- */
    Req_ctx        rctx              ; /* Allocation context.   req_lib.c    */
    Key_lnk        key               ; /* Intern job queues.    req_lib.c    */
    IWORD          lctx              ; /* Low-level device/context index     */
                                       /*                                    */
    Adc_drvdef    *def               ; /* Device generic definitions.        */
    void         (*setfmt       )()  ; /* Configure format of data.          */
    void         (*itsmp        )()  ; /* Enable or disable interrupts.      */
    int          (*getsmp       )()  ; /* Read a new sample.                 */
                                       /*                                    */
    UINT           reg  [ADC_NBREG]  ; /* Registers                          */
                                       /*                                    */
#ifdef NEW_ADC
    IWORD          dma               ; /* DMA in use (1) or not (0).         */
                                       /*                                    */
                                       /* Input format description --------- */
    IWORD          isz               ; /* Size in bytes of a sample.         */
                                       /*                                    */
                                       /* Output format description -------- */
    IWORD          osz               ; /* Size in bytes of a sample.         */
    IWORD          omsk              ; /* Mask to apply to sample.           */
    UCHAR          oorder       [4]  ; /* Sample byte reordering.            */
                                       /*                                    */
                                       /* Format conversion ---------------- */
    IWORD          sof7              ; /* Offset (shift) to apply to sample. */
#else
    UCHAR          sorder       [4]  ; /* Sample byte reordering.            */
    IWORD          ssz               ; /* Size in bytes of a sample.         */
    IWORD          sof7              ; /* Offset (shift) to apply to sample. */
    IWORD          smsk              ; /* Mask to apply to sample.           */
#endif
                                       /*                                    */
    UCHAR         *pbuf              ; /* Pointer to buffer to fill.         */
    IWORD          nbuf              ; /* Available place in buffer.         */
                                       /*                                    */
  END              Adc_ctx           ; /* ---------------------------------- */


  
/*
    3    Public variables
    _____________________
*/

extern UCHAR       adc_numdrv        ; /* Driver identifier for board access.*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


  void  itadcsmp            (IWORD                                           );
  void  itadcdma            (IWORD                                           );


#endif                                 /* End of CACA                        */

#endif                                 /* End of DRV_ADC_H.                  */
