/*****************************************************************************/
/*                                                                           */
/*  Hyperpanel OS                       File "include/drv_i2c.h"             */
/*  Constants and definitions for I2C driver.                                */
/*                                                                           */
/*  Author : JYA                        Last modification : 12/26/2019       */
/*                                                                           */
/*  (c) Copyright Hyperpanel Lab, All Rights Reserved                        */
/*  IDDN.FR.001.320001.*.S.P.1990.000.10000                                  */
/*                                                                           */
/*****************************************************************************/


/*  Some explanations ========================================================


         1  Defines
         2  Structures and typedefs
            2.1  "i2c_io" parameter - I2c_io
            2.2  I2C low-level - I2c_desc/I2c_drvdef/I2c_softdef
            2.3  I2C bus context - I2c_ctx
         3  Public variables
         4  Prototypes of public procedures
         5  RAM data to be included in board_xyz.c


*/


#ifndef DRV_I2C_H                      /* If this file is not already        */
#define DRV_I2C_H                      /* included, include it.              */


/*
    1    Defines
    ____________
*/

/* Bit for the "putI2C_xyz" low-level "flags" parameter .....................*/

#define FLG_START         0x01         /* Generate "start"                   */
#define FLG_STOP          0x02         /* Generate "stop"                    */
#define FLG_NO_ACK        0x04         /* No acknowledgement expected        */



/*
    2    Structures and typedefs
    ____________________________
*/


/*
   2.1  "i2c_io" parameter - I2c_io
   --------------------------------
*/

typedef struct                         /* "i2c_io" list item                 */
  BEGIN                                /* ---------------------------------- */
    UCHAR          io                ; /* I2C address or 0xFF (wait)         */
    UCHAR          flags             ; /* Write flags                        */
    UCHAR          *adr              ; /* Transmit or receive address        */
    USHORT         lg                ; /* Count of bytes to read or write    */
  END              I2c_io            ; /* ---------------------------------- */


/*
   2.2  I2C low-level - I2c_desc/I2c_drvdef/I2c_softdef
   ----------------------------------------------------
*/

typedef struct                         /* Low-level driver procedures addr   */
  BEGIN                                /* ---------------------------------- */
    CHAR           *name             ; /* Device name                        */
    void            (*reset)   ()    ; /* Initialization                     */
    void            (*put)     ()    ; /* Write to the bus                   */
    void            (*get)     ()    ; /* Read from the bus                  */
  END               I2c_desc         ; /* ---------------------------------- */

typedef struct                         /* Generic I2C driver parameters      */
  BEGIN                                /* ---------------------------------- */
    INT             wait             ; /* Wait duration between 2 requests   */
  END               I2c_drvdef       ; /* ---------------------------------- */

typedef struct                         /* I2C software low-level parameters  */
  BEGIN                                /* ---------------------------------- */
    UCHAR           pio_port         ; /* Numero du port PIO des fils I2C    */
    UCHAR           sda_pio          ; /* Numero du PIO du fil SDA           */
    UCHAR           scl_pio          ; /* Numero du PIO du fil SCL           */
    USHORT          hold             ; /* Temps de maintien en us            */
    USHORT          setup            ; /* Temps de chargement en us          */
  END               I2c_softdef      ; /* ---------------------------------- */


/*
   2.3  I2C bus context - I2c_ctx
   ------------------------------
*/


typedef struct                         /* Control loop structure             */
  BEGIN                                /* ---------------------------------- */
    USHORT          cnt              ; /* Loop counter                       */
    UCHAR           begloop          ; /* Start of loop index                */
  END               I2c_loop         ; /* ---------------------------------- */

#define MLOOP      3

typedef struct                         /* I2C context                        */
  BEGIN                                /* ---------------------------------- */
    void            (*reset)    ()   ; /* Controller reset    procedure      */
    void            (*put)      ()   ; /* Send one or N bytes procedure      */
    void            (*get)      ()   ; /* Bus byte read       procedure      */
                                       /*                                    */
    IWORD           sreset           ; /* RESET steps counter                */
    UINT            mode             ; /* mode, 0 if disabled                */
    UINT            speed            ; /* I2C bus speed in KHz               */
    INT             wait             ; /* Wait duration between 2 requests   */
                                       /*                                    */
    SHORT           code             ; /* Code of the REQ_.. event           */
    UCHAR           ad               ; /* I2C address or -1                  */
    UCHAR           flags            ; /* Write flags                        */
    INT             subad            ; /* Register selection sub-address     */
    UCHAR           *frame           ; /* Data buffer address (read/write)   */
    INT             lg               ; /* Count of bytes to be read          */
    UCHAR           maxtry           ; /* Maximum tries count                */
    UCHAR           eepsubsz         ; /* EEPROM subad size in bits          */
    UCHAR           rowsize          ; /* EEPROM row/page size in bytes      */
    UCHAR           wasize           ; /* EEPROM Word Address size in byte   */
    SHORT           raut             ; /* Automaton to be answered to        */
    USHORT          rvl              ; /* Logical way to be answered         */
                                       /*                                    */
    UCHAR           *fracur          ; /* Read/write pointer                 */
    IWORD           try              ; /* Tries counter                      */
    IWORD           tryadr           ; /* Tries counter for the I2C address  */
    UINT            idto             ; /* Current running TO_COM identifier  */
    USHORT          et               ; /* "ti2c" automaton state             */
    USHORT          ev               ; /* "ti2c" automaton event             */
                                       /*                                    */
    I2c_loop        loop[MLOOP]      ; /* Stack for intricated loops         */
    UCHAR           ploop            ; /* Stack pointer for "loop[]"         */
    UCHAR           ioidx            ; /* Index in the I2c_io array/list     */
                                       /*                                    */
    UCHAR           msg[130]         ; /* Register slection message          */
  END               I2c_ctx          ; /* ---------------------------------- */


/*
    3    Public variables
    _____________________
*/


/*
    4    Prototypes of public procedures
    ____________________________________
*/


  IWORD i2c_set             (INT,INT                                         );
  IWORD i2c_send            (INT,INT,UCHAR*,INT,INT                          );
  IWORD i2c_receive         (INT,INT,UCHAR*,INT,INT,INT                      );
  IWORD i2c_write           (INT,INT,UCHAR*,INT                              );
  IWORD i2c_read            (INT,INT,UCHAR*,INT,INT                          );
  IWORD i2c_io              (INT,I2c_io*,INT,INT                             );


#endif                                 /* End of DRV_I2C_H.                  */

/*
    5  RAM data to be included in board_xyz.c
    _________________________________________
*/

#ifdef BOARD_XYZ_DATA_C

const UCHAR     i2c_ctxmax = NUM_I2C ; /* Count of contexts                  */
I2c_ctx         i2c_ctx [NUM_I2C]    ; /* Contexts                           */
USHORT volatile i2c_etat[NUM_I2C]    ; /* Logical Ways states                */


#endif                                 /* End of BOARD_XYZ_C                 */

